#!/bin/bash

cd /home/cahlen/Websites/cahlen.org/

echo "PROCESS ARGUMENTS"
VIDEOIN="${1}"
IMAGEIN="${2}"
TEXTIN="${3}"
TITLE="${4}"
DESC="${5}"
LURL="${6}"
LNAME="${7}"
TAG1="${8}"
TAG2="${9}"
TAG3="${10}"
TAG4="${11}"
TAG5="${12}"

echo "SET GLOBAL VARIABLES"
YEAR="$(date "+%Y")"
MONTH="$(date "+%m")"
DAY="$(date "+%d")"
HOUR="$(date "+%H")"
MINUTE="$(date "+%M")"
SECOND="$(date "+%S")"
TNAME="CahlenLee_""$YEAR""$MONTH""$DAY""_""$TITLE"
FNAME=$(echo "$TNAME" | sed 's/ //g')
URL=$(echo "${TITLE,,}" | sed 's/ /-/g')
HNAME="$YEAR""-""$MONTH""-""$DAY""-""$URL"
FILE="./${HNAME}.html"
VIDEOOUT="./jekyll/_site/files/videos/$YEAR/$MONTH/$FNAME/$FNAME.mp4"
IMAGEOUT="./jekyll/_site/files/videos/$YEAR/$MONTH/$FNAME/$FNAME.jpg"
FOLDEROUT="./jekyll/_site/files/videos/$YEAR/$MONTH/$FNAME"
TEXTOUT="./jekyll/_site/files/videos/$YEAR/$MONTH/$FNAME/$FNAME.txt"

echo "CREATE DIRECTORY"
mkdir ./jekyll/_site/files/videos/"$YEAR"
mkdir ./jekyll/_site/files/videos/"$YEAR"/"$MONTH"
mkdir ./jekyll/_site/files/videos/"$YEAR"/"$MONTH"/"$FNAME"

echo "PROCESS FILES"
cp $VIDEOIN $VIDEOOUT
cp $IMAGEIN $IMAGEOUT
cp $TEXTIN $TEXTOUT

echo "GET IPFS CID"
FLDCID=$(ipfs add --cid-version 1 -r -Q $FOLDEROUT) || { echo "$FOLDEROUT"; echo 'Folder CID failed' ; exit 1; }
JPGCID=$(ipfs add --cid-version 1 --quiet $IMAGEOUT) || { echo 'Jpeg CID failed' ; exit 1; }
MP4CID=$(ipfs add --cid-version 1 --quiet $VIDEOOUT) || { echo 'MP4 CID failed' ; exit 1; } 

echo "WRITE HTML"
/bin/cat <<EOM >$FILE
---
layout: post
title:  "$TITLE"
date:   $YEAR-$MONTH-$DAY $HOUR:$MINUTE:$SECOND -0600
tags: $TAG1
categories: Blog
permalink: /$URL/
image: /files/videos/$YEAR/$MONTH/$FNAME/$FNAME.jpg
video: /files/videos/$YEAR/$MONTH/$FNAME/$FNAME.mp4
description: $DESC
---
<video controls preload="none" width="100%" height="auto" poster="{{ page.image }}" src="{{ page.video }}"></video>
<hr>
<p>{{ page.description }}</p>
<!--more-->
<p>
<a href="$LURL">$LNAME</a>
</p>
EOM

echo "BUILD SITE"
mv ${HNAME}.html ./jekyll/_posts
cd ./jekyll
bundle exec jekyll build

echo "BACKUP ZIP"
cd /home/cahlen/Websites/
zip -r cahlen.org/jekyll/_site/files/misc/source.zip cahlen.org -x */_site/* -x */hive/* -x */dtube/* -x */lbry/* -x */video.sh

echo "SITE TO IPFS"
IPFS="$(ipfs add --cid-version 1 --hidden -Q -r _site)"
ipfs name publish $IPFS
/bin/cat <<EOM >/home/cahlen/Websites/cahlen.org/.ipfs.html
<html>
<head>
</head>
<body>
<p style="color:white;font-size:9px;text-align:center;">
IPFS: <a style="color: darkorange" href="http://$IPFS.ipfs.localhost:8080/" target="_blank">$IPFS</a><br>
IPNS: <a style="color: darkorange" href="http://k51qzi5uqu5dhxa7rv7d5rmuy8qh2alj3ohudyu3g6klsjmsbkovn4r3rcow9p.ipns.localhost:8080/" target="_blank">k51qzi5uqu5dhxa7rv7d5rmuy8qh2alj3ohudyu3g6klsjmsbkovn4r3rcow9p</a>
</p>
</body>
</html>
EOM
ipfs name publish --key=1 "$(ipfs add --cid-version 1 -Q /home/cahlen/Websites/cahlen.org/.ipfs.html)"

echo "WRITE NEWSLETTER"
cd /home/cahlen/Websites/cahlen.org/
/bin/cat <<EOM >"./tmp.txt"
## [$TITLE](https://cahlen.org/$URL/@TrackLink)
[![](https://cahlen.org/files/videos/$YEAR/$MONTH/$FNAME/$FNAME.jpg)](https://cahlen.org/$URL/@TrackLink)

$DESC

[Watch Video](https://cahlen.org/$URL/@TrackLink)

---

EOM
cat ./newsletter.txt >> ./tmp.txt
mv ./tmp.txt ./newsletter.txt

echo "https://ipfs.cahlen.org/ipfs/$FLDCID/$FNAME.jpg"
echo "https://ipfs.cahlen.org/ipfs/$FLDCID/$FNAME.mp4"

echo "ALL DONE!"
